package com.slodon.b2b2c.controller.goods.admin;

import com.slodon.b2b2c.core.constant.GoodsCategoryConst;
import com.slodon.b2b2c.core.constant.GoodsConst;
import com.slodon.b2b2c.core.constant.StoreConst;
import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.JsonResult;
import com.slodon.b2b2c.core.response.PageVO;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.core.response.SldResponse;
import com.slodon.b2b2c.core.util.AssertUtil;
import com.slodon.b2b2c.core.util.UserUtil;
import com.slodon.b2b2c.core.util.WebUtil;
import com.slodon.b2b2c.goods.example.*;
import com.slodon.b2b2c.goods.pojo.*;
import com.slodon.b2b2c.model.goods.*;
import com.slodon.b2b2c.model.seller.StoreBindCategoryModel;
import com.slodon.b2b2c.model.seller.StoreInnerLabelModel;
import com.slodon.b2b2c.model.seller.StoreLabelBindGoodsModel;
import com.slodon.b2b2c.model.seller.StoreModel;
import com.slodon.b2b2c.seller.example.StoreBindCategoryExample;
import com.slodon.b2b2c.seller.example.StoreInnerLabelExample;
import com.slodon.b2b2c.seller.example.StoreLabelBindGoodsExample;
import com.slodon.b2b2c.seller.pojo.*;
import com.slodon.b2b2c.vo.goods.*;
import com.slodon.b2b2c.vo.seller.StoreCategoryTreeVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.map.HashedMap;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static com.slodon.b2b2c.core.constant.GoodsConst.TEMPLATE_CONTENT_1;
import static com.slodon.b2b2c.core.constant.GoodsConst.TEMPLATE_CONTENT_2;

@Api(tags = "admin产品管理")
@RestController
@RequestMapping("v3/goods/admin/seller")
public class GoodsAdminSellerController {

    //seller商品分类
    @Resource
    private GoodsCategoryModel goodsCategoryModel;
    @Resource
    private StoreBindCategoryModel storeBindCategoryModel;
    @Resource
    private StoreModel storeModel;

    //seller关联版式
    @Resource
    private GoodsRelatedTemplateModel goodsRelatedTemplateModel;

    //seller-商品管理
    @Resource
    private GoodsModel goodsModel;
    @Resource
    private GoodsExtendModel goodsExtendModel;
    @Resource
    private GoodsBindLabelModel goodsBindLabelModel;
    @Resource
    private ProductModel productModel;
    @Resource
    private GoodsPictureModel goodsPictureModel;
    @Resource
    private StoreLabelBindGoodsModel storeLabelBindGoodsModel;

    //seller分类绑定信息
    @Resource
    private GoodsAttributeModel goodsAttributeModel;
    @Resource
    private GoodsAttributeValueModel goodsAttributeValueModel;
    @Resource
    private GoodsCategoryBindAttributeModel goodsCategoryBindAttributeModel;
    @Resource
    private GoodsCategoryBindBrandModel goodsCategoryBindBrandModel;
    @Resource
    private GoodsBrandModel goodsBrandModel;

    //seller-属性分组
    @Resource
    private GoodsParameterGroupModel goodsParameterGroupModel;
    @Resource
    private GoodsParameterModel goodsParameterModel;

    //seller-店铺分类管理
    @Resource
    private StoreInnerLabelModel storeInnerLabelModel;

    //seller-商品标签
    @Resource
    private GoodsLabelModel goodsLabelModel;

    //seller规格管理
    @Resource
    private GoodsSpecModel goodsSpecModel;
    @Resource
    private GoodsSpecValueModel goodsSpecValueModel;

    //seller物流运费模板
    @Resource
    private GoodsFreightTemplateModel goodsFreightTemplateModel;
    @Resource
    private GoodsFreightExtendModel goodsFreightExtendModel;

    @ApiOperation("获取发布商品分类列表")
    @GetMapping("cateList")
    public JsonResult<List<GoodsCategoryListVO>> cateList(HttpServletRequest request,
                                                          @RequestParam(value = "storeId") Long storeId) {
        // Vendor vendor = UserUtil.getUser(request, Vendor.class);

        //获取店铺信息
        Store storeDb = storeModel.getStoreByStoreId(storeId);
        AssertUtil.notNull(storeDb, "未获取到店铺信息");
        //根据店铺类型获取分类
        List<GoodsCategoryListVO> vos = new ArrayList<>();
        List<GoodsCategory> thirdList = new ArrayList<>();
        if (storeDb.getIsOwnStore().equals(StoreConst.IS_OWN_STORE)) {
            //自营店铺，获取所有可用的分类
            //获取一级分类列表
            GoodsCategoryExample goodsCategoryExample = new GoodsCategoryExample();
            goodsCategoryExample.setState(GoodsCategoryConst.CATEGORY_STATE_1);
            goodsCategoryExample.setGrade(1);
            goodsCategoryExample.setOrderBy("sort asc");
            List<GoodsCategory> cates1 = goodsCategoryModel.getGoodsCategoryList(goodsCategoryExample, null);
            //遍历一级分类列表，根据pid获取二级分类列表
            if (!CollectionUtils.isEmpty(cates1)) {
                for (GoodsCategory goodsCategory : cates1) {
                    GoodsCategoryListVO vo = new GoodsCategoryListVO(goodsCategory);
                    GoodsCategoryExample goodsCategoryExample2 = new GoodsCategoryExample();
                    goodsCategoryExample2.setPid(goodsCategory.getCategoryId());
                    goodsCategoryExample2.setGrade(2);
                    goodsCategoryExample2.setState(GoodsCategoryConst.CATEGORY_STATE_1);
                    goodsCategoryExample2.setOrderBy("sort asc");
                    List<GoodsCategory> cates2 = goodsCategoryModel.getGoodsCategoryList(goodsCategoryExample2, null);
                    List<GoodsCategoryListVO> secondvos = new ArrayList<>();
                    if (!CollectionUtils.isEmpty(cates2)) {
                        for (GoodsCategory category : cates2) {
                            GoodsCategoryListVO secondvo = new GoodsCategoryListVO(category);
                            GoodsCategoryExample goodsCategoryExample3 = new GoodsCategoryExample();
                            goodsCategoryExample3.setPid(category.getCategoryId());
                            goodsCategoryExample3.setGrade(3);
                            goodsCategoryExample3.setState(GoodsCategoryConst.CATEGORY_STATE_1);
                            goodsCategoryExample3.setOrderBy("sort asc");
                            List<GoodsCategory> cates3 = goodsCategoryModel.getGoodsCategoryList(goodsCategoryExample3, null);
                            List<GoodsCategoryListVO> thirdvos = new ArrayList<>();
                            if (!CollectionUtils.isEmpty(cates3)) {
                                for (GoodsCategory category3 : cates3) {
                                    GoodsCategoryListVO thirdvo = new GoodsCategoryListVO(category3);
                                    thirdvos.add(thirdvo);
                                }
                            }
                            secondvo.setChildren(thirdvos);
                            secondvos.add(secondvo);
                        }
                    }
                    vo.setChildren(secondvos);
                    vos.add(vo);
                }
            }
        } else {
            //入驻店铺，获取审核通过的分类
            StoreBindCategoryExample example = new StoreBindCategoryExample();
            example.setStoreId(storeDb.getStoreId());
            example.setState(StoreConst.STORE_CATEGORY_STATE_PASS);
            List<StoreBindCategory> storeBindCategoryList = storeBindCategoryModel.getStoreBindCategoryList(example, null);
            if (!CollectionUtils.isEmpty(storeBindCategoryList)) {
                for (StoreBindCategory storeBindCategory : storeBindCategoryList) {
                    GoodsCategory category3 = goodsCategoryModel.getGoodsCategoryByCategoryId(storeBindCategory.getGoodsCategoryId3());
                    thirdList.add(category3);
                }
            }
            if (!CollectionUtils.isEmpty(thirdList)) {
                Map<Integer, List<GoodsCategoryListVO>> secondMap = new HashedMap();
                thirdList.forEach(goodsCategory -> {
                    if (secondMap.containsKey(goodsCategory.getPid())) {
                        secondMap.get(goodsCategory.getPid()).add(new GoodsCategoryListVO(goodsCategory));
                    } else {
                        List<GoodsCategoryListVO> thirdVos = new ArrayList<>();
                        thirdVos.add(new GoodsCategoryListVO(goodsCategory));
                        secondMap.put(goodsCategory.getPid(), thirdVos);
                    }
                });
                List<GoodsCategoryListVO> secondVos = new ArrayList<>();
                secondMap.forEach((cateId2, cate3List) -> {
                    GoodsCategory goodsCategory2 = goodsCategoryModel.getGoodsCategoryByCategoryId(cateId2);
                    GoodsCategoryListVO vo = new GoodsCategoryListVO(goodsCategory2);
                    vo.setChildren(cate3List);
                    secondVos.add(vo);
                });
                //一级id和二级列表
                Map<Integer, List<GoodsCategoryListVO>> thirdMap = new HashedMap();
                secondVos.forEach(categoryListVO -> {
                    if (thirdMap.containsKey(categoryListVO.getPid())) {
                        thirdMap.get(categoryListVO.getPid()).add(categoryListVO);
                    } else {
                        ArrayList<GoodsCategoryListVO> listVOS = new ArrayList<>();
                        listVOS.add(categoryListVO);
                        thirdMap.put(categoryListVO.getPid(), listVOS);
                    }
                });
                thirdMap.forEach((cateId1, cate2List) -> {
                    GoodsCategory goodCategory1 = goodsCategoryModel.getGoodsCategoryByCategoryId(cateId1);
                    GoodsCategoryListVO vo = new GoodsCategoryListVO(goodCategory1);
                    vo.setChildren(cate2List);
                    vos.add(vo);
                });
            }
        }
        return SldResponse.success(vos);
    }

    @ApiOperation("关联版式列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "templateName", value = "版式名称", paramType = "query"),
            @ApiImplicitParam(name = "templatePosition", value = "版式位置", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", defaultValue = "20", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面位置", defaultValue = "1", paramType = "query")
    })
    @GetMapping("bsList")
    public JsonResult<PageVO<GoodsRelatedTemplateVO>> getBsList(HttpServletRequest request,
                                                              @RequestParam(value = "templateName", required = false) String templateName,
                                                              @RequestParam(value = "templatePosition", required = false) Integer templatePosition,
                                                              @RequestParam(value = "storeId") Long storeId) {
        // Vendor vendor = UserUtil.getUser(request, Vendor.class);
        //分页设置
        PagerInfo pager = WebUtil.handlerPagerInfo(request);

        //根据条件查询集合
        GoodsRelatedTemplateExample example = new GoodsRelatedTemplateExample();
        example.setStoreId(storeId);
        example.setTemplateNameLike(templateName);
        if (templatePosition != null && (templatePosition == TEMPLATE_CONTENT_1 || templatePosition == TEMPLATE_CONTENT_2)) {
            example.setTemplatePosition(templatePosition);
        }
        List<GoodsRelatedTemplate> list = goodsRelatedTemplateModel.getGoodsRelatedTemplateList(example, pager);

        //响应
        List<GoodsRelatedTemplateVO> vos = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list)) {
            list.forEach(goodsRelatedTemplate -> {
                GoodsRelatedTemplateVO vo = new GoodsRelatedTemplateVO(goodsRelatedTemplate);
                vos.add(vo);
            });
        }
        return SldResponse.success(new PageVO<>(vos, pager));
    }

    @ApiOperation("获取商品信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "goodsId", value = "商品Id", paramType = "query", required = true),
    })
    @GetMapping("getGoodsInfo")
    public JsonResult<SellerGoodsDetailVO> getGoodsInfo(HttpServletRequest request,
                                                        @RequestParam(value = "goodsId") Long goodsId) {
        Goods goods = goodsModel.getGoodsByGoodsId(goodsId);
        if (goods == null) {
            throw new MallException("商品不存在，请重试！");
        }
        GoodsExtend goodsExtend = goodsExtendModel.getGoodsExtendByGoodsId(goodsId);
        if (goodsExtend == null) {
            throw new MallException("商品不存在，请重试！");
        }
        //商品绑定标签
        GoodsBindLabelExample goodsBindLabelExample = new GoodsBindLabelExample();
        goodsBindLabelExample.setGoodsId(goodsId);
        goodsBindLabelExample.setOrderBy("bind_id asc");
        List<GoodsBindLabel> goodsBindLabelList = goodsBindLabelModel.getGoodsBindLabelList(goodsBindLabelExample, null);

        //商品绑定店铺内部分类
        StoreLabelBindGoodsExample storeLabelBindGoodsExample = new StoreLabelBindGoodsExample();
        storeLabelBindGoodsExample.setGoodsId(goodsId);
        storeLabelBindGoodsExample.setOrderBy("bind_id asc");
        List<StoreLabelBindGoods> storeLabelBindGoodsList = storeLabelBindGoodsModel.getStoreLabelBindGoodsList(storeLabelBindGoodsExample, null);

        //正常和禁用的货品
        ProductExample productExample = new ProductExample();
        productExample.setGoodsId(goodsId);
        productExample.setStateIn(GoodsConst.PRODUCT_STATE_1 + "," + GoodsConst.PRODUCT_STATE_2);
        productExample.setOrderBy("product_id asc");
        List<Product> productList = productModel.getProductList(productExample, null);

        //查询商品图片
        GoodsPictureExample goodsPictureExample = new GoodsPictureExample();
        goodsPictureExample.setGoodsId(goodsId);
        goodsPictureExample.setOrderBy("picture_id asc");
        List<GoodsPicture> goodsPictureList = goodsPictureModel.getGoodsPictureList(goodsPictureExample, null);


        SellerGoodsDetailVO sellerGoodsDetailVO = new SellerGoodsDetailVO(goods, goodsExtend, goodsBindLabelList, storeLabelBindGoodsList, goodsPictureList, productList);
        return SldResponse.success(sellerGoodsDetailVO);
    }

    @ApiOperation("分类绑定的属性,品牌列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "categoryId", value = "分类Id", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "20", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "1", paramType = "query")
    })
    @GetMapping("listByCategoryId")
    public JsonResult<GoodsCategoryBindVO> getFlList(HttpServletRequest request,
                                                   @RequestParam(value = "categoryId", required = false) Integer categoryId) {
        AssertUtil.isTrue(categoryId < 1, "分类Id错误");
        GoodsCategoryBindVO goodsCategoryBindVO = new GoodsCategoryBindVO(getBrandList(categoryId), getAttributeList(categoryId));
        return SldResponse.success(goodsCategoryBindVO);
    }
    /**
     * 查询分类绑定的属性列表
     *
     * @param categoryId
     * @return
     */
    private List<GoodsAttributeVO> getAttributeList(Integer categoryId) {
        //查分类绑定的属性表
        GoodsCategoryBindAttributeExample goodsCategoryBindAttributeExample = new GoodsCategoryBindAttributeExample();
        goodsCategoryBindAttributeExample.setCategoryId(categoryId);
        List<GoodsCategoryBindAttribute> goodsCategoryBindAttributeList = goodsCategoryBindAttributeModel.getGoodsCategoryBindAttributeList(goodsCategoryBindAttributeExample, null);
        if (CollectionUtils.isEmpty(goodsCategoryBindAttributeList)) {
            return new ArrayList<>();
        }

        StringBuffer attributeIds = new StringBuffer();
        goodsCategoryBindAttributeList.forEach(goodsCategoryBindAttribute -> {
            attributeIds.append(",").append(goodsCategoryBindAttribute.getAttributeId());
        });
        //查属性表中的对应属性
        GoodsAttributeExample goodsAttributeExample = new GoodsAttributeExample();
        goodsAttributeExample.setAttributeIdIn(attributeIds.toString().substring(1));
        goodsAttributeExample.setIsShow(GoodsConst.IS_ATTRIBUTE_YES);
        goodsAttributeExample.setOrderBy("sort asc, create_time desc");
        List<GoodsAttribute> goodsAttributeList = goodsAttributeModel.getGoodsAttributeList(goodsAttributeExample, null);
        if (CollectionUtils.isEmpty(goodsAttributeList)) {
            return new ArrayList<>();
        }

        List<GoodsAttributeVO> vos = new ArrayList<>();

        goodsAttributeList.forEach(attribute -> {
            //查属性值表
            GoodsAttributeValueExample goodsAttributeValueExample = new GoodsAttributeValueExample();
            goodsAttributeValueExample.setAttributeId(attribute.getAttributeId());
            List<GoodsAttributeValue> goodsAttributeValueList = goodsAttributeValueModel.getGoodsAttributeValueList(goodsAttributeValueExample, null);

            GoodsAttributeVO vo = new GoodsAttributeVO(attribute, goodsAttributeValueList);
            vos.add(vo);
        });

        return vos;
    }

    /**
     * 查询分类绑定的品牌
     *
     * @param categoryId
     * @return
     */
    private List<GoodsBrandVO> getBrandList(Integer categoryId) {
        GoodsCategoryBindBrandExample goodsCategoryBindBrandExample = new GoodsCategoryBindBrandExample();
        goodsCategoryBindBrandExample.setCategoryId(categoryId);
        List<GoodsCategoryBindBrand> goodsCategoryBindBrandList = goodsCategoryBindBrandModel.getGoodsCategoryBindBrandList(goodsCategoryBindBrandExample, null);

        if (CollectionUtils.isEmpty(goodsCategoryBindBrandList)) {
            //未绑定品牌
            return new ArrayList<>();
        }

        StringBuffer brandIds = new StringBuffer();
        goodsCategoryBindBrandList.forEach(goodsCategoryBindBrand -> {
            brandIds.append(",").append(goodsCategoryBindBrand.getBrandId());
        });

        GoodsBrandExample goodsBrandExample = new GoodsBrandExample();
        goodsBrandExample.setBrandIdIn(brandIds.toString().substring(1));
        List<GoodsBrand> brandList = goodsBrandModel.getGoodsBrandList(goodsBrandExample, null);
        List<GoodsBrandVO> vos = new ArrayList<>();
        if (!CollectionUtils.isEmpty(brandList)) {
            brandList.forEach(brand -> {
                GoodsBrandVO vo = new GoodsBrandVO(brand);
                vos.add(vo);
            });
        }
        return vos;
    }

    @ApiOperation("可用属性分组列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageSize", value = "分页大小", defaultValue = "20", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面位置", defaultValue = "1", paramType = "query")
    })
    @GetMapping("canUseList")
    public JsonResult<PageVO<GoodsParameterGroupVO>> canUseList(HttpServletRequest request,@RequestParam(value = "storeId") Long storeId) {
        // Vendor vendor = UserUtil.getUser(request, Vendor.class);

        //分页设置
        PagerInfo pager = WebUtil.handlerPagerInfo(request);
        //根据条件查询集合
        GoodsParameterGroupExample example = new GoodsParameterGroupExample();
        example.setStoreId(storeId);
        example.setIsShow(GoodsConst.IS_ATTRIBUTE_YES);
        example.setOrderBy("sort asc, create_time desc");
        List<GoodsParameterGroup> list = goodsParameterGroupModel.getGoodsParameterGroupList(example, pager);
        List<GoodsParameterGroupVO> vos = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list)) {
            list.forEach(goodsParameterGroup -> {
                GoodsParameterExample parameterExample = new GoodsParameterExample();
                parameterExample.setStoreId(storeId);
                parameterExample.setGroupId(goodsParameterGroup.getGroupId());
                parameterExample.setIsShow(GoodsConst.IS_ATTRIBUTE_YES);
                List<GoodsParameter> parameterList = goodsParameterModel.getGoodsParameterList(parameterExample, null);
                if (CollectionUtils.isEmpty(parameterList)) {
                    pager.setRowsCount(pager.getRowsCount() - 1);
                } else {
                    GoodsParameterGroupVO vo = new GoodsParameterGroupVO(goodsParameterGroup);
                    vos.add(vo);
                }
            });
        }
        return SldResponse.success(new PageVO<>(vos, pager));
    }

    @ApiOperation("获取店铺分类列表")
    @GetMapping("dpList")
    public JsonResult getDpList(HttpServletRequest request,@RequestParam(value = "storeId") Long storeId) {
        PagerInfo pager = WebUtil.handlerPagerInfo(request);
        // Vendor vendor = UserUtil.getUser(request, Vendor.class);

        List<StoreCategoryTreeVO> tree = new ArrayList<>();
        StoreInnerLabelExample storeInnerLabelExample = new StoreInnerLabelExample();
        storeInnerLabelExample.setParentInnerLabelId(0);
        storeInnerLabelExample.setStoreId(storeId);
        List<StoreInnerLabel> storeInnerLabelList = storeInnerLabelModel.getStoreInnerLabelList(storeInnerLabelExample, pager);
        generateTree(tree, storeInnerLabelList, 2);

        return SldResponse.success(tree);
    }

    /**
     * 递归生成树
     *
     * @param treeList
     * @param data
     * @return
     */
    private List<StoreCategoryTreeVO> generateTree(List<StoreCategoryTreeVO> treeList, List<StoreInnerLabel> data, Integer grade) {
        if (grade > 0) {
            for (StoreInnerLabel storeInnerLabel : data) {
                StoreCategoryTreeVO tree = new StoreCategoryTreeVO();
                tree.setInnerLabelId(storeInnerLabel.getInnerLabelId());
                tree.setInnerLabelName(storeInnerLabel.getInnerLabelName());
                tree.setInnerLabelSort(storeInnerLabel.getInnerLabelSort());
                tree.setIsShow(storeInnerLabel.getIsShow());
                tree.setParentInnerLabelId(storeInnerLabel.getParentInnerLabelId());
                tree.setCreateTime(storeInnerLabel.getCreateTime());
                tree.setUpdateTime(storeInnerLabel.getUpdateTime());

                StoreInnerLabelExample storeInnerLabelExample = new StoreInnerLabelExample();
                storeInnerLabelExample.setParentInnerLabelId(storeInnerLabel.getInnerLabelId());
                List<StoreCategoryTreeVO> vos = generateTree(new ArrayList<>(),
                        storeInnerLabelModel.getStoreInnerLabelList(storeInnerLabelExample, null), grade - 1);
                if (!CollectionUtils.isEmpty(vos)) {
                    tree.setChildren(vos);
                } else {
                    tree.setChildren(null);
                }
                treeList.add(tree);
            }
        }
        return treeList;
    }

    @ApiOperation("商品标签列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "labelName", value = "标签名称", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", defaultValue = "20", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面位置", defaultValue = "1", paramType = "query")
    })
    @GetMapping("bqList")
    public JsonResult<PageVO<GoodsLabelVO>> getBqList(HttpServletRequest request, String labelName) {
        //分页设置
        PagerInfo pager = WebUtil.handlerPagerInfo(request);

        //根据标签名称查询商品标签集合
        GoodsLabelExample example = new GoodsLabelExample();
        example.setLabelNameLike(labelName);
        example.setOrderBy("sort asc, create_time desc");
        List<GoodsLabel> list = goodsLabelModel.getGoodsLabelList(example, pager);

        ArrayList<GoodsLabelVO> vos = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list)) {
            list.forEach(goodsLabel -> {
                GoodsLabelVO vo = new GoodsLabelVO(goodsLabel);
                vos.add(vo);
            });
        }
        return SldResponse.success(new PageVO<>(vos, pager));
    }

    @ApiOperation("规格列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "specName", value = "规则名称", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "20", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "1", paramType = "query")
    })
    @GetMapping("ggList")
    public JsonResult<PageVO<SellerGoodsSpecVO>> getGgList(HttpServletRequest request,
                                                           @RequestParam(value = "storeId") Long storeId,
                                                            @RequestParam(value = "specName", required = false) String specName) {
        // Vendor vendor = UserUtil.getUser(request, Vendor.class);
        PagerInfo pager = WebUtil.handlerPagerInfo(request);
        GoodsSpecValueExample goodsSpecValueExample = new GoodsSpecValueExample();
        GoodsSpecExample goodsSpecExample = new GoodsSpecExample();
        goodsSpecExample.setSpecNameLike(specName);
        goodsSpecExample.setStoreIdOrZero(storeId);
        goodsSpecExample.setPager(pager);
        List<GoodsSpec> goodsSpecList = goodsSpecModel.getGoodsSpecList(goodsSpecExample, pager);
        List<SellerGoodsSpecVO> vos = new ArrayList<>();
        if (!CollectionUtils.isEmpty(goodsSpecList)) {
            goodsSpecList.forEach(spec -> {
                goodsSpecValueExample.setSpecId(spec.getSpecId());
                List<GoodsSpecValue> goodsSpecValueList = goodsSpecValueModel.getGoodsSpecValueList(goodsSpecValueExample, null);
                SellerGoodsSpecVO vo = new SellerGoodsSpecVO(spec, goodsSpecValueList);
                vos.add(vo);
            });
        }
        return SldResponse.success(new PageVO<>(vos, goodsSpecExample.getPager()));
    }

    @ApiOperation("分类列表,获取当前分类的下级分类，0代表获取所有1级分类")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "categoryId", value = "分类Id", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "20", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "1", paramType = "query")
    })
    @GetMapping("list")
    public JsonResult<PageVO<GoodsCategoryListVO>> getList(HttpServletRequest request,
                                                           @RequestParam(value = "categoryId", required = true) Integer categoryId) {
        PagerInfo pager = WebUtil.handlerPagerInfo(request);
        GoodsCategoryExample goodsCategoryExample = new GoodsCategoryExample();
        goodsCategoryExample.setPid(categoryId);
        goodsCategoryExample.setPager(pager);
        goodsCategoryExample.setOrderBy("sort asc");
        List<GoodsCategory> goodsCategoryList = goodsCategoryModel.getGoodsCategoryList(goodsCategoryExample, pager);
        List<GoodsCategoryListVO> vos = new ArrayList<>();
        AssertUtil.notNull(goodsCategoryList, "未查询到分类");
        for (GoodsCategory goodsCategory : goodsCategoryList) {
            vos.add(new GoodsCategoryListVO(goodsCategory));
        }
        return SldResponse.success(new PageVO<>(vos, goodsCategoryExample.getPager()));
    }

    @ApiOperation("获取运费模板列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "templateName", value = "模板名称", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", defaultValue = "20", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面位置", defaultValue = "1", paramType = "query")
    })
    @GetMapping("yfList")
    public JsonResult<PageVO<GoodsFreightTemplateVO>> getYfList(HttpServletRequest request,
                                                                @RequestParam(value = "storeId") Long storeId,
                                                                @RequestParam(value = "templateName", required = false) String templateName) {
        PagerInfo pager = WebUtil.handlerPagerInfo(request);
        // Vendor vendor = UserUtil.getUser(request, Vendor.class);

        GoodsFreightTemplateExample goodsFreightTemplateExample = new GoodsFreightTemplateExample();
        goodsFreightTemplateExample.setStoreId(storeId);
        goodsFreightTemplateExample.setTemplateNameLike(templateName);
        List<GoodsFreightTemplate> goodsFreightTemplateList = goodsFreightTemplateModel.getGoodsFreightTemplateList(goodsFreightTemplateExample, pager);
        List<GoodsFreightTemplateVO> vos = new ArrayList<>();
        if (!CollectionUtils.isEmpty(goodsFreightTemplateList)) {
            for (GoodsFreightTemplate goodsFreightTemplate : goodsFreightTemplateList) {
                GoodsFreightExtendExample extendExample = new GoodsFreightExtendExample();
                extendExample.setFreightTemplateId(goodsFreightTemplate.getFreightTemplateId());
                goodsFreightTemplate.setFreightExtendList(goodsFreightExtendModel.getGoodsFreightExtendList(extendExample, null));
                GoodsFreightTemplateVO vo = new GoodsFreightTemplateVO(goodsFreightTemplate);
                vos.add(vo);
            }
        }
        return SldResponse.success(new PageVO<>(vos, pager));
    }
}
