DROP TABLE IF EXISTS live_broadcast;
CREATE TABLE live_broadcast  (
  roomId varchar(50) NOT NULL COMMENT '房间ID' ,
  name varchar(50) NOT NULL COMMENT '直播间名字，最短3个汉字，最长17个汉字，1个汉字相当于2个字符',
  coverImg varchar(50) NOT NULL COMMENT '背景图，填入mediaID（mediaID获取后，三天内有效）；图片 mediaID 的获取，请参考以下文档： https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/
  New_temporary_materials.html；直播间背景图，图片规则：建议像素1080*1920，大小不超过2M',
  coverImgPath varchar(50) NOT NULL COMMENT '背景图 本地相对路径',
  startTime bigint(20) NOT NULL COMMENT '直播计划开始时间（开播时间需要在当前时间的10分钟后 并且 开始时间不能在 6 个月后）',
  endTime bigint(20) NOT NULL COMMENT '直播计划结束时间（开播时间和结束时间间隔不得短于30分钟，不得超过24小时）',
  anchorName varchar(50) NOT NULL COMMENT '主播昵称，最短2个汉字，最长15个汉字，1个汉字相当于2个字符',
  anchorWechat varchar(50) NOT NULL COMMENT '主播微信号，如果未实名认证，需要先前往“小程序直播”小程序进行实名验证, 小程序二维码链接：https://res.wx.qq.com/op_res/9rSix1dhHfK4rR049JL0PHJ7TpOvkuZ3mE0z7Ou_Etvjf-w1J_jVX0rZqeStLfwh',
  subAnchorWechat varchar(50)  COMMENT '主播副号微信号，如果未实名认证，需要先前往“小程序直播”小程序进行实名验证, 小程序二维码链接：https://res.wx.qq.com/op_res/9rSix1dhHfK4rR049JL0PHJ7TpOvkuZ3mE0z7Ou_Etvjf-w1J_jVX0rZqeStLfwh',
  createrWechat varchar(50)  COMMENT '创建者微信号，不传入则此直播间所有成员可见。传入则此房间仅创建者、管理员、超管、直播间主播可见',
  shareImg varchar(50) NOT NULL COMMENT '分享图，填入mediaID（mediaID获取后，三天内有效）；图片 mediaID 的获取，请参考以下文档： https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/New_temporary_materials.html；直播间分享图，图片规则：建议像素800*640，大小不超过1M；',
  shareImgPath varchar(50) NOT NULL COMMENT '分享图 本地相对路径',
  feedsImg varchar(50) NOT NULL COMMENT '购物直播频道封面图，填入mediaID（mediaID获取后，三天内有效）；图片 mediaID 的获取，请参考以下文档： https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/New_temporary_materials.html; 购物直播频道封面图，图片规则：建议像素800*800，大小不超过100KB；',
  feedsImgPath varchar(50) NOT NULL COMMENT '购物直播频道封面图 本地相对路径',
  isFeedsPublic int(11) NOT NULL DEFAULT 1 COMMENT '是否开启官方收录 【1: 开启，0：关闭】，默认开启收录',
  type  int(11) NOT NULL  COMMENT '直播间类型 【1: 推流，0：手机直播】',
  closeLike int(11) NOT NULL  COMMENT '是否关闭点赞 【0：开启，1：关闭】（若关闭，观众端将隐藏点赞按钮，直播开始后不允许开启）',
  closeGoods  int(11) NOT NULL  COMMENT '是否关闭货架 【0：开启，1：关闭】（若关闭，观众端将隐藏商品货架，直播开始后不允许开启）',
  closeComment  int(11) NOT NULL  COMMENT '是否关闭评论 【0：开启，1：关闭】（若关闭，观众端将隐藏评论入口，直播开始后不允许开启）',
  closeReplay int(11) NOT NULL DEFAULT 1 COMMENT '是否关闭回放 【0：开启，1：关闭】默认关闭回放（直播开始后允许开启）',
  closeShare  int(11) NOT NULL DEFAULT 0 COMMENT '是否关闭分享 【0：开启，1：关闭】默认开启分享（直播开始后不允许修改）',
  closeKf int(11) NOT NULL DEFAULT 1 COMMENT '是否关闭客服 【0：开启，1：关闭】 默认关闭客服（直播开始后允许开启）',
  liveStatus int(11) NOT NULL DEFAULT 102 COMMENT '直播间状态。101：直播中，102：未开始，103已结束，104禁播，105：暂停，106：异常，107：已过期',
  PRIMARY KEY (roomId) USING BTREE
) ENGINE = InnoDB AUTO_INCREMENT = 1 CHARACTER SET = utf8 COLLATE = utf8_general_ci COMMENT = '直播信息表' ROW_FORMAT = Dynamic;

DROP TABLE IF EXISTS live_goods;
CREATE TABLE live_goods  (
  goodsId varchar(50) NOT NULL COMMENT '微信返回商品ID' ,
  auditId varchar(50) NOT NULL COMMENT '审核单ID' ,
  name varchar(50) NOT NULL COMMENT '商品名称，最长14个汉字，1个汉字相当于2个字符',
  coverImgUrl varchar(300) NOT NULL COMMENT '填入mediaID（mediaID获取后，三天内有效）；图片 mediaID 的获取，请参考以下文档： https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/New_temporary_materials.html；图片规则：图片尺寸最大300像素*300像素；',
  priceType int(11) NOT NULL DEFAULT 1  COMMENT '价格类型，1：一口价（只需要传入price，price2不传） 2：价格区间（price字段为左边界，price2字段为右边界，price和price2必传） 3：显示折扣价（price字段为原价，price2字段为现价， price和price2必传）',
  price varchar(50) NOT NULL COMMENT '数字，最多保留两位小数，单位元',
  price2 varchar(50) COMMENT '数字，最多保留两位小数，单位元',
  url varchar(300) NOT NULL COMMENT '商品详情页的小程序路径，路径参数存在 url 的，该参数的值需要进行 encode 处理再填入',
  thirdPartyAppid varchar(50)  COMMENT '当商品为第三方小程序的商品则填写为对应第三方小程序的appid',
  auditStatus int(11) NOT NULL DEFAULT 1 COMMENT '商品间状态。0：未审核，1：审核中，2:审核通过，3审核失败',
  localGoodsId varchar(50) NOT NULL COMMENT '本地goods表商品ID' ,
  vendorId varchar(50) NOT NULL COMMENT '商户ID' ,
  PRIMARY KEY (goodsId) USING BTREE
) ENGINE = InnoDB AUTO_INCREMENT = 1 CHARACTER SET = utf8 COLLATE = utf8_general_ci COMMENT = '直播商品信息表' ROW_FORMAT = Dynamic;

DROP TABLE IF EXISTS live_goods_relation;
CREATE TABLE live_goods_relation  (
  roomId varchar(50) NOT NULL COMMENT '房间ID' ,
  goodsId varchar(50) NOT NULL COMMENT '微信返回商品ID'
) ENGINE = InnoDB AUTO_INCREMENT = 1 CHARACTER SET = utf8 COLLATE = utf8_general_ci COMMENT = '直播商品关联表' ROW_FORMAT = Dynamic;